@@ -6,42 +6,104 @@ from __future__ import (division, print_function, absolute_import,
 
 __all__ = ["PTSampler"]
 
-try:
-    import acor
-    acor = acor
-except ImportError:
-    acor = None
-
-from .sampler import Sampler
-import multiprocessing as multi
 import numpy as np
 import numpy.random as nr
+import multiprocessing as multi
+
+from . import autocorr
+from .sampler import Sampler
+
+def default_beta_ladder(ndim, ntemps=None, Tmax=None):
+    """Returns a ladder of :math:`\beta \equiv 1/T` with temperatures
+    geometrically spaced with spacing chosen so that a Gaussian
+    posterior would have a 0.25 temperature swap acceptance rate.
+
+    :param ndim:
+        The number of dimensions in the parameter space.
+
+    :param ntemps: (optional)
+        If set, the number of temperatures to use.  If ``None``, the
+        ``Tmax`` argument must be given, and the number of
+        temperatures is chosen so that the highest temperature is
+        greater than ``Tmax``.
+
+    :param Tmax: (optional)
+        If ``ntemps`` is not given, this argument controls the number
+        of temperatures.  Temperatures are chosen according to the
+        spacing criteria until the maximum temperature exceeds
+        ``Tmax``
+
+    """
+    tstep = np.array([25.2741, 7., 4.47502, 3.5236, 3.0232,
+                      2.71225, 2.49879, 2.34226, 2.22198, 2.12628,
+                      2.04807, 1.98276, 1.92728, 1.87946, 1.83774,
+                      1.80096, 1.76826, 1.73895, 1.7125, 1.68849,
+                      1.66657, 1.64647, 1.62795, 1.61083, 1.59494,
+                      1.58014, 1.56632, 1.55338, 1.54123, 1.5298,
+                      1.51901, 1.50881, 1.49916, 1.49, 1.4813,
+                      1.47302, 1.46512, 1.45759, 1.45039, 1.4435,
+                      1.4369, 1.43056, 1.42448, 1.41864, 1.41302,
+                      1.40761, 1.40239, 1.39736, 1.3925, 1.38781,
+                      1.38327, 1.37888, 1.37463, 1.37051, 1.36652,
+                      1.36265, 1.35889, 1.35524, 1.3517, 1.34825,
+                      1.3449, 1.34164, 1.33847, 1.33538, 1.33236,
+                      1.32943, 1.32656, 1.32377, 1.32104, 1.31838,
+                      1.31578, 1.31325, 1.31076, 1.30834, 1.30596,
+                      1.30364, 1.30137, 1.29915, 1.29697, 1.29484,
+                      1.29275, 1.29071, 1.2887, 1.28673, 1.2848,
+                      1.28291, 1.28106, 1.27923, 1.27745, 1.27569,
+                      1.27397, 1.27227, 1.27061, 1.26898, 1.26737,
+                      1.26579, 1.26424, 1.26271, 1.26121,
+                      1.25973])
+    dmax = tstep.shape[0]
+        
+    if ndim > dmax:
+        # An approximation to the temperature step at large
+        # dimension
+        tstep = 1.0 + 2.0*np.sqrt(np.log(4.0))/np.sqrt(ndim)
+    else:
+        tstep = tstep[ndim-1]
+        
+    if ntemps is None and Tmax is None:
+        raise ValueError('must specify one of ``ntemps`` and ``Tmax``')
+    elif ntemps is None:
+        ntemps = int(np.log(Tmax)/np.log(tstep)+2)
+
+    return np.exp(np.linspace(0, -(ntemps-1)*np.log(tstep), ntemps))
 
 
 class PTLikePrior(object):
-    """Wrapper class for logl and logp.
+    """
+    Wrapper class for logl and logp.
 
     """
 
-    def __init__(self, logl, logp):
+    def __init__(self, logl, logp, loglargs=[], logpargs=[], loglkwargs={},
+                 logpkwargs={}):
         self.logl = logl
         self.logp = logp
+        self.loglargs = loglargs
+        self.logpargs = logpargs
+        self.loglkwargs = loglkwargs
+        self.logpkwargs = logpkwargs
 
     def __call__(self, x):
-        lp = self.logp(x)
+        lp = self.logp(x, *self.logpargs, **self.logpkwargs)
 
         if lp == float('-inf'):
             return lp, lp
 
-        return self.logl(x), lp
+        return self.logl(x, *self.loglargs, **self.loglkwargs), lp
 
 
 class PTSampler(Sampler):
-    """A parallel-tempered ensemble sampler, using :class:`EnsembleSampler`
+    """
+    A parallel-tempered ensemble sampler, using :class:`EnsembleSampler`
     for sampling within each parallel chain.
 
     :param ntemps:
-        The number of temperatures.
+        The number of temperatures.  Can be ``None``, in which case
+        the ``Tmax`` argument sets the maximum temperature.
 
     :param nwalkers:
         The number of ensemble walkers at each temperature.
@@ -72,16 +134,43 @@ class PTSampler(Sampler):
     :param a: (optional)
         Proposal scale factor.
 
+    :param Tmax: (optional)
+        Maximum temperature for the ladder.  If ``ntemps`` is
+        ``None``, this argument is used to set the temperature ladder.
+
+    :param loglargs: (optional)
+        Positional arguments for the log-likelihood function.
+
+    :param logpargs: (optional)
+        Positional arguments for the log-prior function.
+
+    :param loglkwargs: (optional)
+        Keyword arguments for the log-likelihood function.
+
+    :param logpkwargs: (optional)
+        Keyword arguments for the log-prior function.
+
     """
     def __init__(self, ntemps, nwalkers, dim, logl, logp, threads=1,
-                 pool=None, betas=None, a=2.0):
+                 pool=None, betas=None, a=2.0, Tmax=None, loglargs=[], logpargs=[],
+                 loglkwargs={}, logpkwargs={}):
         self.logl = logl
         self.logp = logp
         self.a = a
+        self.loglargs = loglargs
+        self.logpargs = logpargs
+        self.loglkwargs = loglkwargs
+        self.logpkwargs = logpkwargs
 
-        self.ntemps = ntemps
         self.nwalkers = nwalkers
         self.dim = dim
+        
+        if betas is None:
+            self._betas = default_beta_ladder(self.dim, ntemps=ntemps, Tmax=Tmax)
+        else:
+            self._betas = betas
+
+        self.ntemps = self.betas.shape[0]
 
         assert self.nwalkers % 2 == 0, \
             "The number of walkers must be even."
@@ -92,13 +181,8 @@ class PTSampler(Sampler):
         self._lnprob = None
         self._lnlikelihood = None
 
-        if betas is None:
-            self._betas = self.default_beta_ladder()
-        else:
-            self._betas = betas
-
-        self.nswap = np.zeros(ntemps, dtype=np.float)
-        self.nswap_accepted = np.zeros(ntemps, dtype=np.float)
+        self.nswap = np.zeros(self.ntemps, dtype=np.float)
+        self.nswap_accepted = np.zeros(self.ntemps, dtype=np.float)
 
         self.nprop = np.zeros((self.ntemps, self.nwalkers), dtype=np.float)
         self.nprop_accepted = np.zeros((self.ntemps, self.nwalkers),
@@ -108,45 +192,6 @@ class PTSampler(Sampler):
         if threads > 1 and pool is None:
             self.pool = multi.Pool(threads)
 
-    def default_beta_ladder(self):
-        """Returns a ladder of :math:`\beta \equiv 1/T` with temperatures
-        geometrically spaced with spacing chosen so that a Gaussian
-        posterior would have a 0.25 temperature swap acceptance rate.
-
-        """
-        tstep = np.array([25.2741, 7., 4.47502, 3.5236, 3.0232,
-                          2.71225, 2.49879, 2.34226, 2.22198, 2.12628,
-                          2.04807, 1.98276, 1.92728, 1.87946, 1.83774,
-                          1.80096, 1.76826, 1.73895, 1.7125, 1.68849,
-                          1.66657, 1.64647, 1.62795, 1.61083, 1.59494,
-                          1.58014, 1.56632, 1.55338, 1.54123, 1.5298,
-                          1.51901, 1.50881, 1.49916, 1.49, 1.4813,
-                          1.47302, 1.46512, 1.45759, 1.45039, 1.4435,
-                          1.4369, 1.43056, 1.42448, 1.41864, 1.41302,
-                          1.40761, 1.40239, 1.39736, 1.3925, 1.38781,
-                          1.38327, 1.37888, 1.37463, 1.37051, 1.36652,
-                          1.36265, 1.35889, 1.35524, 1.3517, 1.34825,
-                          1.3449, 1.34164, 1.33847, 1.33538, 1.33236,
-                          1.32943, 1.32656, 1.32377, 1.32104, 1.31838,
-                          1.31578, 1.31325, 1.31076, 1.30834, 1.30596,
-                          1.30364, 1.30137, 1.29915, 1.29697, 1.29484,
-                          1.29275, 1.29071, 1.2887, 1.28673, 1.2848,
-                          1.28291, 1.28106, 1.27923, 1.27745, 1.27569,
-                          1.27397, 1.27227, 1.27061, 1.26898, 1.26737,
-                          1.26579, 1.26424, 1.26271, 1.26121,
-                          1.25973])
-        dmax = tstep.shape[0]
-
-        if self.dim > dmax:
-            # An approximation to the temperature step at large
-            # dimension
-            tstep = 1.0 + 2.0*np.sqrt(np.log(4.0))/np.sqrt(self.dim)
-        else:
-            tstep = tstep[self.dim-1]
-
-        return np.exp(np.linspace(0, -(self.ntemps-1)*np.log(tstep),
-                                  self.ntemps))
-
     def reset(self):
         """
         Clear the ``chain``, ``lnprobability``, ``lnlikelihood``,
@@ -206,7 +251,8 @@ class PTSampler(Sampler):
 
         # If we have no lnprob or logls compute them
         if lnprob0 is None or lnlike0 is None:
-            fn = PTLikePrior(self.logl, self.logp)
+            fn = PTLikePrior(self.logl, self.logp, self.loglargs,
+                             self.logpargs, self.loglkwargs, self.logpkwargs)
             if self.pool is None:
                 results = list(map(fn, p.reshape((-1, self.dim))))
             else:
@@ -259,8 +305,7 @@ class PTSampler(Sampler):
                 pupdate = p[:, jupdate::2, :]
                 psample = p[:, jsample::2, :]
 
-                us = np.random.uniform(size=(self.ntemps, self.nwalkers/2))
-                zs = np.square(1.0 + (self.a-1.0)*us)/self.a
+                zs = np.exp(np.random.uniform(low=-np.log(self.a), high=np.log(self.a), size=(self.ntemps, self.nwalkers/2)))
 
                 qs = np.zeros((self.ntemps, self.nwalkers/2, self.dim))
                 for k in range(self.ntemps):
@@ -270,7 +315,9 @@ class PTSampler(Sampler):
                         (self.nwalkers / 2, 1)) * (pupdate[k, :, :] -
                                                    psample[k, js, :])
 
-                fn = PTLikePrior(self.logl, self.logp)
+                fn = PTLikePrior(self.logl, self.logp, self.loglargs,
+                                 self.logpargs, self.loglkwargs,
+                                 self.logpkwargs)
                 if self.pool is None:
                     results = list(map(fn, qs.reshape((-1, self.dim))))
                 else:
@@ -284,7 +331,7 @@ class PTSampler(Sampler):
                 qslnprob = qslogls * self.betas.reshape((self.ntemps, 1)) \
                     + qslogps
 
-                logpaccept = (self.dim-1)*np.log(zs) + qslnprob \
+                logpaccept = self.dim*np.log(zs) + qslnprob \
                     - lnprob[:, jupdate::2]
                 logrs = np.log(np.random.uniform(low=0.0, high=1.0,
                                                  size=(self.ntemps,
@@ -448,6 +495,17 @@ class PTSampler(Sampler):
         """
         return self._chain
 
+    @property
+    def flatchain(self):
+        """Returns the stored chain, but flattened along the walker axis, so
+        of shape ``(Ntemps, Nwalkers*Nsteps, Ndim)``.
+
+        """
+
+        s = self.chain.shape
+
+        return self._chain.reshape((s[0], -1, s[3]))
+
     @property
     def lnprobability(self):
         """
@@ -489,13 +547,21 @@ class PTSampler(Sampler):
         parameter in each temperature of shape ``(Ntemps, Ndim)``.
 
         """
-        if acor is None:
-            raise ImportError('acor')
-        else:
-            acors = np.zeros((self.ntemps, self.dim))
+        return self.get_autocorr_time()
 
-            for i in range(self.ntemps):
-                for j in range(self.dim):
-                    acors[i, j] = acor.acor(self._chain[i, :, :, j])[0]
+    def get_autocorr_time(self, window=50):
+        """
+        Returns a matrix of autocorrelation lengths for each
+        parameter in each temperature of shape ``(Ntemps, Ndim)``.
 
-            return acors
+        :param window: (optional)
+            The size of the windowing function. This is equivalent to the
+            maximum number of lags to use. (default: 50)
+
+        """
+        acors = np.zeros((self.ntemps, self.dim))
+
+        for i in range(self.ntemps):
+            x = np.mean(self._chain[i, :, :, :], axis=0)
+            acors[i, :] = autocorr.integrated_time(x, window=window)
+        return acors
